@@ -89,6 +89,10 @@
 #' * If `train_N` is a vector, then its size must be `N` even the re-fit frequency specified in `freq` is not one.
 #'
 #' Defaults to `100`.
+#' @param refit_cores the number of cores/workers to be used to re-fit GP components (in the same layer of a DGP emulator)
+#'     at each M-step during the re-fitting. If set to `NULL`, the number of cores is set to `(max physical cores available - 1)`.
+#'     Only use multiple cores when there is a large number of GP components in different layers and optimization of GP components
+#'     is computationally expensive. Defaults to `1`.
 #' @param ... any arguments (with names different from those of arguments used in [design()]) that are used by `f`, `method`, and `eval`
 #'     can be passed here. [design()] will pass relevant arguments to `f`, `method`, and `eval` based on the names of additional arguments provided.
 #'
@@ -426,9 +430,9 @@ design.gp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, lim
   } else {
     add_arg <- list(...)
     xy_cand_list <- check_xy_cand(x_cand, y_cand, n_dim_Y)
+    xy_cand_list <- remove_dup(xy_cand_list, X)
     x_cand <- xy_cand_list[[1]]
     y_cand <- xy_cand_list[[2]]
-    x_cand <- remove_dup(x_cand, X)
     idx_x_cand0 <- c(1:nrow(x_cand))
     idx_x_cand <- idx_x_cand0
     idx_x_acq <- c()
@@ -631,12 +635,16 @@ design.gp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, lim
 #' @rdname design
 #' @method design dgp
 #' @export
-design.dgp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, limits = NULL, int = FALSE, f = NULL, freq = c(1, 1), x_test = NULL, y_test = NULL, target = NULL, method = mice, eval = NULL, verb = TRUE, check_point = NULL, cores = 1, train_N = 100, ...) {
+design.dgp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, limits = NULL, int = FALSE, f = NULL, freq = c(1, 1), x_test = NULL, y_test = NULL, target = NULL, method = mice, eval = NULL, verb = TRUE, check_point = NULL, cores = 1, train_N = 100, refit_cores = 1, ...) {
   if ( !inherits(object,"dgp") ) stop("'object' must be an instance of the 'dgp' class.", call. = FALSE)
 
   N <- check_N(N)
   freq <- check_freq(freq)
   train_N <- check_train_N(train_N, N)
+  if( !is.null(refit_cores) ) {
+    refit_cores <- as.integer(refit_cores)
+    if ( refit_cores < 1 ) stop("'refit_cores' must be >= 1.", call. = FALSE)
+  }
   n_cand <- check_n_cand(n_cand)
   if (!is.null(x_test) & !is.null(y_test)) {
     xy_test <- check_xy_test(x_test, y_test)
@@ -802,7 +810,7 @@ design.dgp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, li
 
         if ( i %% freq[1]==0 | i==N){
           if ( verb ) message(" - Updating and re-fitting ...", appendLF = FALSE)
-          object <- update(object, X, Y, refit = TRUE, verb = FALSE, N = train_N[i], B = 10)
+          object <- update(object, X, Y, refit = TRUE, verb = FALSE, N = train_N[i], cores = refit_cores, B = 10)
           if ( verb ) message(" done")
         } else {
           if ( verb ) message(" - Updating ...", appendLF = FALSE)
@@ -865,9 +873,9 @@ design.dgp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, li
   } else {
     add_arg <- list(...)
     xy_cand_list <- check_xy_cand(x_cand, y_cand, n_dim_Y)
+    xy_cand_list <- remove_dup(xy_cand_list, X)
     x_cand <- xy_cand_list[[1]]
     y_cand <- xy_cand_list[[2]]
-    x_cand <- remove_dup(x_cand, X)
     idx_x_cand0 <- c(1:nrow(x_cand))
     idx_x_cand <- idx_x_cand0
     idx_x_acq <- c()
@@ -972,7 +980,7 @@ design.dgp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, li
 
         if ( i %% freq[1]==0 | i==N ){
           if ( verb ) message(" - Updating and re-fitting ...", appendLF = FALSE)
-          object <- update(object, X, Y, refit = TRUE, verb = FALSE, N = train_N[i], B = 10)
+          object <- update(object, X, Y, refit = TRUE, verb = FALSE, N = train_N[i], cores = refit_cores, B = 10)
           if ( verb ) message(" done")
         } else {
           if ( verb ) message(" - Updating ...", appendLF = FALSE)
@@ -1082,12 +1090,16 @@ design.dgp <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, li
 #' @rdname design
 #' @method design bundle
 #' @export
-design.bundle <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, limits = NULL, int = FALSE, f = NULL, freq = c(1, 1), x_test = NULL, y_test = NULL, target = NULL, method = mice, eval = NULL, verb = TRUE, check_point = NULL, cores = 1, train_N = 100, ...) {
+design.bundle <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200, limits = NULL, int = FALSE, f = NULL, freq = c(1, 1), x_test = NULL, y_test = NULL, target = NULL, method = mice, eval = NULL, verb = TRUE, check_point = NULL, cores = 1, train_N = 100, refit_cores = 1, ...) {
   if ( !inherits(object,"bundle") ) stop("'object' must be an instance of the 'bundle' class.", call. = FALSE)
 
   N <- check_N(N)
   freq <- check_freq(freq)
   train_N <- check_train_N(train_N, N)
+  if( !is.null(refit_cores) ) {
+    refit_cores <- as.integer(refit_cores)
+    if ( refit_cores < 1 ) stop("'refit_cores' must be >= 1.", call. = FALSE)
+  }
   n_cand <- check_n_cand(n_cand)
   if (!is.null(x_test) & !is.null(y_test)) {
     xy_test <- check_xy_test(x_test, y_test)
@@ -1357,13 +1369,13 @@ design.bundle <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200,
             if ( is.null(target) ) {
               obj_k <- object[[paste('emulator',k,sep='')]]
               if ( inherits(obj_k,"gp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE)
-              if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], B = 10)
+              if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], cores = refit_cores, B = 10)
               object[[paste('emulator',k,sep='')]] <- obj_k
             } else {
               if ( !istarget[k] ){
                 obj_k <- object[[paste('emulator',k,sep='')]]
                 if ( inherits(obj_k,"gp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE)
-                if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], B = 10)
+                if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], cores = refit_cores, B = 10)
                 object[[paste('emulator',k,sep='')]] <- obj_k
               }
             }
@@ -1486,12 +1498,12 @@ design.bundle <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200,
   } else {
     add_arg <- list(...)
     xy_cand_list <- check_xy_cand(x_cand, y_cand, n_emulators)
-    x_cand <- xy_cand_list[[1]]
-    y_cand <- xy_cand_list[[2]]
 
     for (j in 1:n_emulators){
-      x_cand <- remove_dup(x_cand, X[[paste('emulator',j,sep="")]])
+      xy_cand_list <- remove_dup(xy_cand_list, X[[paste('emulator',j,sep="")]])
     }
+    x_cand <- xy_cand_list[[1]]
+    y_cand <- xy_cand_list[[2]]
     idx_x_cand0 <- c(1:nrow(x_cand))
     idx_x_cand <- idx_x_cand0
     idx_x_acq <- c()
@@ -1667,13 +1679,13 @@ design.bundle <- function(object, N, x_cand = NULL, y_cand = NULL, n_cand = 200,
             if ( is.null(target) ){
               obj_k <- object[[paste('emulator',k,sep='')]]
               if ( inherits(obj_k,"gp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE)
-              if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], B = 10)
+              if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], cores = refit_cores, B = 10)
               object[[paste('emulator',k,sep='')]] <- obj_k
             } else {
               if ( !istarget[k] ){
                 obj_k <- object[[paste('emulator',k,sep='')]]
                 if ( inherits(obj_k,"gp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE)
-                if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], B = 10)
+                if ( inherits(obj_k,"dgp") ) obj_k <- update(obj_k, X[[paste('emulator',k,sep="")]], Y[[paste('emulator',k,sep="")]], refit = TRUE, verb = FALSE, N = train_N[i], cores = refit_cores, B = 10)
                 object[[paste('emulator',k,sep='')]] <- obj_k
               }
             }
@@ -1879,14 +1891,18 @@ check_xy_cand <- function(x_cand, y_cand, n_dim_Y){
 }
 
 #remove duplicates between x_cand and X
-remove_dup <- function(x_cand, X){
+remove_dup <- function(xy_cand_list, X){
+  x_cand <- xy_cand_list[[1]]
+  y_cand <- xy_cand_list[[2]]
   X_s <- apply(X, 1, paste, collapse = ", ")
   x_cand_s <- apply(x_cand, 1, paste, collapse = ", ")
   X_x_cand <- intersect(X_s, x_cand_s)
   if ( length(X_x_cand)!=0 ){
-    x_cand <- x_cand[!x_cand_s %in% X_x_cand,,drop=FALSE]
+    idx <- !x_cand_s %in% X_x_cand
+    x_cand <- x_cand[idx,,drop=FALSE]
+    y_cand <- y_cand[idx,,drop=FALSE]
   }
-  return(x_cand)
+  return(list(x_cand, y_cand))
 }
 
 #check argument x_test and y_test
