@@ -3,6 +3,7 @@ pkg.env$dgpsi <- NULL
 pkg.env$py_buildin <- NULL
 pkg.env$np <- NULL
 pkg.env$copy <- NULL
+pkg.env$py_gc <- NULL
 
 #' @title 'python' environment initialization
 #'
@@ -39,10 +40,10 @@ init_py <- function(py_ver = NULL, dgpsi_ver = NULL, reinstall = FALSE, uninstal
     #dgpsi_ver <- c('cython>=0.29.30', 'dill>=0.3.2', 'jupyter>=1.0.0', 'matplotlib-base>=3.2.1', 'numba >=0.51.2',
     #               'numpy >=1.18.2', 'pathos >=0.2.9', 'psutil >=5.8.0', 'pybind11 >=2.10.0', 'pythran >=0.11.0',
     #               'scikit-build >=0.15.0', 'scikit-learn >=0.22.0', 'scipy >=1.4.1', 'tqdm >=4.50.2', 'tabulate >=0.8.7')
-    #env_name <- 'dgp_si_R_2_1_5_9000'
+    #env_name <- 'dgp_si_R_2_1_6_9000'
     ##For release version
-    dgpsi_ver <- 'dgpsi==2.1.6'
-    env_name <- 'dgp_si_R_2_1_6'
+    dgpsi_ver <- 'dgpsi==2.2.0'
+    env_name <- 'dgp_si_R_2_2_0'
   } else {
     env_name <- paste('dgp_si_R_', gsub(".", "_", dgpsi_ver,fixed=TRUE), sep = "")
     dgpsi_ver <- paste('dgpsi==', dgpsi_ver, sep = "")
@@ -127,6 +128,7 @@ init_py <- function(py_ver = NULL, dgpsi_ver = NULL, reinstall = FALSE, uninstal
     assign('py_buildin', reticulate::import_builtins(), pkg.env)
     assign('np', reticulate::import("numpy"), pkg.env)
     assign('copy', reticulate::import("copy"), pkg.env)
+    assign('py_gc', reticulate::import("gc"), pkg.env)
     message(" done")
     Sys.sleep(0.5)
     message("The Python environment for 'dgpsi' is successfully loaded.")
