@@ -147,6 +147,8 @@ mice.gp <- function(object, x_cand, batch_size = 1, nugget_s = 1e-6, workers = 1
       idx_x_cand <- idx_x_cand0[-idx]
     }
   }
+  pkg.env$py_gc$collect()
+  gc(full=T)
   return(idx)
 }
 
@@ -213,6 +215,7 @@ mice.dgp <- function(object, x_cand, batch_size = 1, nugget_s = 1e-6, workers =
     emulator_obj_cp <- pkg.env$copy$deepcopy(object$emulator_obj)
     B <- as.integer(length(emulator_obj_cp$all_layer_set))
     burnin <- constructor_obj_cp$burnin
+    isblock <- constructor_obj_cp$block
     for (i in 1:batch_size){
       if ( identical(workers,as.integer(1)) ){
         res = emulator_obj_cp$metric(x_cand = x_cand[idx_x_cand,,drop=F], method = 'MICE', nugget_s = nugget_s, score_only = TRUE)
@@ -244,13 +247,15 @@ mice.dgp <- function(object, x_cand, batch_size = 1, nugget_s = 1e-6, workers =
       constructor_obj_cp$update_xy(training_input, training_output)
 
       est_obj <- constructor_obj_cp$estimate(burnin)
-      emulator_obj_cp <- pkg.env$dgpsi$emulator(all_layer = est_obj, N = B)
+      emulator_obj_cp <- pkg.env$dgpsi$emulator(all_layer = est_obj, N = B, block = isblock)
 
       idx <- c(idx,  idx_x_cand[idx_i])
       idx_x_cand <- idx_x_cand0[-unique(idx)]
     }
     idx <- matrix(idx, nrow = batch_size, byrow = T)
   }
+  pkg.env$py_gc$collect()
+  gc(full=T)
   return(idx)
 }
 
@@ -376,9 +381,10 @@ mice.bundle <- function(object, x_cand, batch_size = 1, nugget_s = 1e-6, workers
         } else {
           B <- as.integer(length(obj_j$emulator_obj$all_layer_set))
           burnin <- obj_j$constructor_obj$burnin
+          isblock <- obj_j$constructor_obj$block
           constructor_obj_list[[j]]$update_xy(rbind(training_input[[j]], X_new[j,]), rbind(training_output[[j]], Y_new_j))
           est_obj <- constructor_obj_list[[j]]$estimate(burnin)
-          emulator_obj_list[[j]] <- pkg.env$dgpsi$emulator(all_layer = est_obj, N = B)
+          emulator_obj_list[[j]] <- pkg.env$dgpsi$emulator(all_layer = est_obj, N = B, block = isblock)
         }
       }
       idx <- c(idx,  idx_x_cand[idx_i])
@@ -386,6 +392,8 @@ mice.bundle <- function(object, x_cand, batch_size = 1, nugget_s = 1e-6, workers
     }
     idx <- matrix(idx, nrow = batch_size, byrow = T)
   }
+  pkg.env$py_gc$collect()
+  gc(full=T)
   return(idx)
 }
 
